==> lstdataprocess128-1.3.0.tar.gz: lstdataprocess128-1.3.0/src/lstdpp128/EventData.cpp <==
/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "EventData.h"
#include <sstream>
#include <iomanip>

using namespace std;

namespace lstdpp128 {

inline float floatData(Event const & event, int index) {
	return *reinterpret_cast<const float *>(&event.data[index]);
}

uint32_t eventData(Event const & event) {
	return event.data[0];
}

uint32_t eventDataPlus(Event const & event) {
	return event.data[1];
}

///////////////////////////////////////////////////////////////////////////////
// Energy

bool boardHasEnergy(BoardType type) {

	if (type == CFD
	 || type == C1724_PHA
	 || type == C1730_PHA
	 || type == C1740_QDC
	 || type == C1725_PHA) {
		return true;
	}
	return false;
}

bool eventHasEnergy(Event const & event) {
	return boardHasEnergy(event.boardType());
}

uint16_t boardEnergyResolution(BoardType type) {

	switch (type) {
	case CFD:
		return 12;

	case C1724_PHA:
		return 15;

	case C1730_PHA:
		return 15;

	case C1740_QDC:
		return 16;

	case C1725_PHA:
		return 15;
	}

	return 0;
}

uint16_t eventEnergyResolution(Event const & event) {
	return boardEnergyResolution(event.boardType());
}

uint16_t eventEnergy(Event const & event) {

	if (event.boardType() == CFD) {
		return event.data[0] & 0xFFF;
	}
	else if (event.boardType() == C1724_PHA) {
		return event.data[0] & 0x7FFF;
	}
	else if (event.boardType() == C1730_PHA) {
		return event.data[0] & 0x7FFF;
	}
	else if (event.boardType() == C1740_QDC) {
		return event.data[0] & 0xFFFF;
	}
	else if (event.boardType() == C1725_PHA) {
		return event.data[0] & 0x7FFF;
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Q

bool boardHasQ(BoardType type) {

	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD) {
		return true;
	}
	return false;
}

bool eventHasQ(Event const & event) {
	return boardHasQ(event.boardType());
}

uint16_t boardQShortResolution(BoardType type) {

	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD) {
		return 15;
	}
	return 0;
}

uint16_t eventQShortResolution(Event const & event) {
	return boardQShortResolution(event.boardType());
}

uint16_t boardQLongResolution(BoardType type) {

	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD) {
		return 16;
	}
	return 0;
}

uint16_t eventQLongResolution(Event const & event) {
	return boardQLongResolution(event.boardType());
}

uint16_t eventQShort(Event const & event) {

	// data = qlong 16bits | 1bit | qshort 15bits
	return event.data[0] & 0x7FFF;
}

uint16_t eventQLong(Event const & event) {

	// data = qlong 16bits | 1bit | qshort 15bits
	return (event.data[0] >> 16);

}

float eventQRatio(Event const & event) {

	uint16_t qLong = eventQLong(event);
	if (qLong == 0) {
		return 0.0f;
	}

	return (static_cast<float>(qLong) - static_cast<float>(eventQShort(event))) / static_cast<float>(qLong);
}


///////////////////////////////////////////////////////////////////////////////
// Position XY

bool boardHasPositionX(BoardType type) {

	if (type == C1740_QDIV
	 || type == C1740_QDC
	 || type == MCC
	 || type == CAMERA) {
		return true;
	}
	return false;
}

bool eventHasPositionX(Event const & event) {
	return boardHasPositionX(event.boardType());
}

bool boardHasPositionY(BoardType type) {

	if (type == C1740_QDIV
	 || type == MCC
	 || type == CAMERA) {
		return true;
	}
	return false;
}

bool eventHasPositionY(Event const & event) {
	return boardHasPositionY(event.boardType());
}

uint16_t boardPositionXResolution(BoardType type) {

	switch (type) {
	case C1740_QDIV:
		return 5;

	case C1740_QDC:
		return 6;

	case MCC:
		return 16;

	case CAMERA:
		return 16;
	}

	return 0;
}

uint16_t eventPositionXResolution(Event const & event) {
	return boardPositionXResolution(event.boardType());
}

uint16_t boardPositionYResolution(BoardType type) {

	switch (type) {
	case C1740_QDIV:
		return 8;

	case MCC:
		return 16;

	case CAMERA:
		return 16;
	}

	return 0;
}

uint16_t eventPositionYResolution(Event const & event) {
	return boardPositionYResolution(event.boardType());
}

uint16_t eventPositionX(Event const & event) {

	// data = Position X 16bits | Position Y 16bits
	return (event.data[0] >> 16) & 0xFFFF;
}

uint16_t eventPositionY(Event const & event) {

	// data = Position X 16bits | Position Y 16bits
	return event.data[0] & 0xFFFF;
}


///////////////////////////////////////////////////////////////////////////////
// A, B

bool boardHasAB(BoardType type) {

	if (type == C1740_QDIV) {
		return true;
	}
	return false;
}

bool eventHasAB(Event const & event) {
	return boardHasAB(event.boardType());
}

uint16_t boardAResolution(BoardType type) {

	if (type == C1740_QDIV) {
		return 16;
	}
	return 0;
}

uint16_t eventAResolution(Event const & event) {
	return boardAResolution(event.boardType());
}

uint16_t boardAplusBResolution(BoardType type) {

	if (type == C1740_QDIV) {
		return 16;
	}
	return 0;
}

uint16_t eventAplusBResolution(Event const & event) {
	return boardAplusBResolution(event.boardType());
}

uint16_t eventA(Event const & event) {

	// data = A+B 16bits | A 16bits
	return event.data[1] & 0xFFFF;
}

uint16_t eventAplusB(Event const & event) {

	// data = A+B 16bits | A 16bits
	return (event.data[1] >> 16) & 0xFFFF;
}


///////////////////////////////////////////////////////////////////////////////
// Zero Crossing

bool boardHasZeroCrossing(BoardType type) {
	if (type == CFD) {
		return true;
	}
	return false;
}

bool eventHasZeroCrossing(Event const & event) {
	return boardHasZeroCrossing(event.boardType());
}

uint16_t boardZeroCrossingResolution(BoardType type) {
	if (type == CFD) {
		return 32;
	}
	return 0;
}

uint16_t eventZeroCrossingResolution(Event const & event) {
	return boardZeroCrossingResolution(event.boardType());
}

float eventZeroCrossing(Event const & event) {
	return floatData(event, 1);
}

///////////////////////////////////////////////////////////////////////////////
// Special unspecified data

bool boardHasUnspecifiedData(BoardType type) {
	if (type == GGPICKUP) {
		return true;
	}
	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD
	 || type == C1724_PHA
	 || type == C1730_PHA
	 || type == C1725_PHA
	 || type == C1740_QDC) {
		return true;
	}
	return false;
}

bool eventHasUnspecifiedData(Event const & event) {
	return boardHasUnspecifiedData(event.boardType());
}

uint16_t boardUnspecifiedDataResolution(BoardType type) {
	if (type == GGPICKUP) {
		return 32;
	}
	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD
	 || type == C1724_PHA
	 || type == C1730_PHA
	 || type == C1725_PHA
	 || type == C1740_QDC) {
		return 16;
	}
	return 0;
}

uint32_t eventUnspecifiedDataResolution(Event const & event) {
	return boardUnspecifiedDataResolution(event.boardType());
}

uint32_t eventUnspecifiedData(Event const & event) {
	return event.data[0];
}

///////////////////////////////////////////////////////////////////////////////
// Flags

bool boardHasFlags(BoardType type) {

	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD
	 || type == C1724_PHA
	 || type == C1730_PHA
	 || type == C1725_PHA
	 || type == C1740_QDC) {
		return true;
	}
	return false;
}

bool eventHasFlags(Event const & event) {
	return boardHasFlags(event.boardType());
}

uint16_t eventFlags(Event const & event) {

	if (eventHasFlags(event)) {
		// Shift 16 bits.
		return (event.data[1] >> 16);
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// To string

std::string eventDataColumnsToString(BoardType type) {

	ostringstream os;

	// data[0]
	if (boardHasEnergy(type)) {
		os << setw(12) << "energy";
	}

	if (boardHasQ(type)) {
		os << setw(12) << "q-short";
		os << setw(12) << "q-long";
	}

	if (boardHasAB(type)) {
		os << setw(12) << "A";
		os << setw(12) << "A+B";
	}

	if (boardHasPositionX(type)) {
		os << setw(12) << "X";
	}

	if (boardHasPositionY(type)) {
		os << setw(12) << "X";
	}

	// data[1]
	if (boardHasZeroCrossing(type)) {
		os << setw(12) << "0-crossing";
	}

	// flags.
	if (boardHasFlags(type)) {
		os << setw(18) << "flags";
	}

	return os.str();
}

std::string eventDataToString(Event const & event) {

	ostringstream os;

	// data[0]
	if (eventHasEnergy(event)) {
		os << setw(12) << eventEnergy(event);
	}

	if (eventHasQ(event)) {
		os << setw(12) << eventQShort(event);
		os << setw(12) << eventQLong(event);
	}

	if (eventHasAB(event)) {
		os << setw(12) << eventA(event);
		os << setw(12) << eventAplusB(event);
	}

	if (eventHasPositionX(event)) {
		os << setw(12) << eventPositionX(event);
	}

	if (eventHasPositionY(event)) {
		os << setw(12) << eventPositionY(event);
	}

	// data[1]
	if (eventHasZeroCrossing(event)) {
		os << setw(12) << eventZeroCrossing(event);
	}

	if (eventHasFlags(event)) {

		ostringstream bos;

		uint16_t flags = eventFlags(event);
		for (int i = 15; i >= 0; --i) {
			uint16_t flag = (flags >> i) & 0x1;
			if (flag == 1) {
				bos << "1";
			}
			else {
				bos << "0";
			}
		}

		os << setw(18) << bos.str();
	}

	return os.str();
}

}

==> lstdataprocess128-1.3.2.tar.gz: lstdataprocess128-1.3.2/src/lstdpp128/EventData.cpp <==
/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "EventData.h"
#include <sstream>
#include <iomanip>

using namespace std;

namespace lstdpp128 {

inline float floatData(Event const & event, int index) {
	return *reinterpret_cast<const float *>(&event.data[index]);
}

uint32_t eventData(Event const & event) {
	return event.data[0];
}

uint32_t eventDataPlus(Event const & event) {
	return event.data[1];
}

///////////////////////////////////////////////////////////////////////////////
// Energy

bool boardHasEnergy(BoardType type) {

	if (type == CFD
	 || type == C1724_PHA
	 || type == C1730_PHA
	 || type == C1740_QDC
	 || type == C1725_PHA) {
		return true;
	}
	return false;
}

bool eventHasEnergy(Event const & event) {
	return boardHasEnergy(event.boardType());
}

uint16_t boardEnergyResolution(BoardType type) {

	switch (type) {
	case CFD:
		return 12;

	case C1724_PHA:
		return 15;

	case C1730_PHA:
		return 15;

	case C1740_QDC:
		return 16;

	case C1725_PHA:
		return 15;
	}

	return 0;
}

uint16_t eventEnergyResolution(Event const & event) {
	return boardEnergyResolution(event.boardType());
}

uint16_t eventEnergy(Event const & event) {

	if (event.boardType() == CFD) {
		return event.data[0] & 0xFFF;
	}
	else if (event.boardType() == C1724_PHA) {
		return event.data[0] & 0x7FFF;
	}
	else if (event.boardType() == C1730_PHA) {
		return event.data[0] & 0x7FFF;
	}
	else if (event.boardType() == C1740_QDC) {
		return event.data[0] & 0xFFFF;
	}
	else if (event.boardType() == C1725_PHA) {
		return event.data[0] & 0x7FFF;
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Q

bool boardHasQ(BoardType type) {

	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD) {
		return true;
	}
	return false;
}

bool eventHasQ(Event const & event) {
	return boardHasQ(event.boardType());
}

uint16_t boardQShortResolution(BoardType type) {

	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD) {
		return 15;
	}
	return 0;
}

uint16_t eventQShortResolution(Event const & event) {
	return boardQShortResolution(event.boardType());
}

uint16_t boardQLongResolution(BoardType type) {

	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD) {
		return 16;
	}
	return 0;
}

uint16_t eventQLongResolution(Event const & event) {
	return boardQLongResolution(event.boardType());
}

uint16_t eventQShort(Event const & event) {

	// data = qlong 16bits | 1bit | qshort 15bits
	return event.data[0] & 0x7FFF;
}

uint16_t eventQLong(Event const & event) {

	// data = qlong 16bits | 1bit | qshort 15bits
	return (event.data[0] >> 16);

}

float eventQRatio(Event const & event) {

	uint16_t qLong = eventQLong(event);
	if (qLong == 0) {
		return 0.0f;
	}

	return (static_cast<float>(qLong) - static_cast<float>(eventQShort(event))) / static_cast<float>(qLong);
}


///////////////////////////////////////////////////////////////////////////////
// Position XY

bool boardHasPositionX(BoardType type) {

	if (type == C1740_QDIV
	 || type == C1740_QDC
	 || type == MCC
	 || type == CAMERA) {
		return true;
	}
	return false;
}

bool eventHasPositionX(Event const & event) {
	return boardHasPositionX(event.boardType());
}

bool boardHasPositionY(BoardType type) {

	if (type == C1740_QDIV
	 || type == MCC
	 || type == CAMERA) {
		return true;
	}
	return false;
}

bool eventHasPositionY(Event const & event) {
	return boardHasPositionY(event.boardType());
}

uint16_t boardPositionXResolution(BoardType type) {

	switch (type) {
	case C1740_QDIV:
		return 5;

	case C1740_QDC:
		return 6;

	case MCC:
		return 16;

	case CAMERA:
		return 16;
	}

	return 0;
}

uint16_t eventPositionXResolution(Event const & event) {
	return boardPositionXResolution(event.boardType());
}

uint16_t boardPositionYResolution(BoardType type) {

	switch (type) {
	case C1740_QDIV:
		return 8;

	case MCC:
		return 16;

	case CAMERA:
		return 16;
	}

	return 0;
}

uint16_t eventPositionYResolution(Event const & event) {
	return boardPositionYResolution(event.boardType());
}

uint16_t eventPositionX(Event const & event) {

	// data = Position X 16bits | Position Y 16bits
	return (event.data[0] >> 16) & 0xFFFF;
}

uint16_t eventPositionY(Event const & event) {

	// data = Position X 16bits | Position Y 16bits
	return event.data[0] & 0xFFFF;
}


///////////////////////////////////////////////////////////////////////////////
// A, B

bool boardHasAB(BoardType type) {

	if (type == C1740_QDIV) {
		return true;
	}
	return false;
}

bool eventHasAB(Event const & event) {
	return boardHasAB(event.boardType());
}

uint16_t boardAResolution(BoardType type) {

	if (type == C1740_QDIV) {
		return 16;
	}
	return 0;
}

uint16_t eventAResolution(Event const & event) {
	return boardAResolution(event.boardType());
}

uint16_t boardAplusBResolution(BoardType type) {

	if (type == C1740_QDIV) {
		return 16;
	}
	return 0;
}

uint16_t eventAplusBResolution(Event const & event) {
	return boardAplusBResolution(event.boardType());
}

uint16_t eventA(Event const & event) {

	// data = A+B 16bits | A 16bits
	return event.data[1] & 0xFFFF;
}

uint16_t eventAplusB(Event const & event) {

	// data = A+B 16bits | A 16bits
	return (event.data[1] >> 16) & 0xFFFF;
}


///////////////////////////////////////////////////////////////////////////////
// Zero Crossing

bool boardHasZeroCrossing(BoardType type) {
	if (type == CFD) {
		return true;
	}
	return false;
}

bool eventHasZeroCrossing(Event const & event) {
	return boardHasZeroCrossing(event.boardType());
}

uint16_t boardZeroCrossingResolution(BoardType type) {
	if (type == CFD) {
		return 32;
	}
	return 0;
}

uint16_t eventZeroCrossingResolution(Event const & event) {
	return boardZeroCrossingResolution(event.boardType());
}

float eventZeroCrossing(Event const & event) {
	return floatData(event, 1);
}

///////////////////////////////////////////////////////////////////////////////
// Special unspecified data

bool boardHasUnspecifiedData(BoardType type) {
	if (type == GGPICKUP
	 || type == GGDMS
	 || type == GGFMCPICKUP) {
		return true;
	}
	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD
	 || type == C1724_PHA
	 || type == C1730_PHA
	 || type == C1725_PHA
	 || type == C1740_QDC) {
		return true;
	}
	return false;
}

bool eventHasUnspecifiedData(Event const & event) {
	return boardHasUnspecifiedData(event.boardType());
}

uint16_t boardUnspecifiedDataResolution(BoardType type) {
	if (type == GGPICKUP
	 || type == GGDMS
	 || type == GGFMCPICKUP) {
		return 32;
	}
	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD
	 || type == C1724_PHA
	 || type == C1730_PHA
	 || type == C1725_PHA
	 || type == C1740_QDC) {
		return 16;
	}
	return 0;
}

uint32_t eventUnspecifiedDataResolution(Event const & event) {
	return boardUnspecifiedDataResolution(event.boardType());
}

uint32_t eventUnspecifiedData(Event const & event) {
	return event.data[0];
}

///////////////////////////////////////////////////////////////////////////////
// Flags

bool boardHasFlags(BoardType type) {

	if (type == C1751_PSD
	 || type == C1730_PSD
	 || type == C1725_PSD
	 || type == C1724_PHA
	 || type == C1730_PHA
	 || type == C1725_PHA
	 || type == C1740_QDC) {
		return true;
	}
	return false;
}

bool eventHasFlags(Event const & event) {
	return boardHasFlags(event.boardType());
}

uint16_t eventFlags(Event const & event) {

	if (eventHasFlags(event)) {
		// Shift 16 bits.
		return (event.data[1] >> 16);
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// To string

std::string eventDataColumnsToString(BoardType type) {

	ostringstream os;

	// data[0]
	if (boardHasEnergy(type)) {
		os << setw(12) << "energy";
	}

	if (boardHasQ(type)) {
		os << setw(12) << "q-short";
		os << setw(12) << "q-long";
	}

	if (boardHasAB(type)) {
		os << setw(12) << "A";
		os << setw(12) << "A+B";
	}

	if (boardHasPositionX(type)) {
		os << setw(12) << "X";
	}

	if (boardHasPositionY(type)) {
		os << setw(12) << "X";
	}

	// data[1]
	if (boardHasZeroCrossing(type)) {
		os << setw(12) << "0-crossing";
	}

	// flags.
	if (boardHasFlags(type)) {
		os << setw(18) << "flags";
	}

	return os.str();
}

std::string eventDataToString(Event const & event) {

	ostringstream os;

	// data[0]
	if (eventHasEnergy(event)) {
		os << setw(12) << eventEnergy(event);
	}

	if (eventHasQ(event)) {
		os << setw(12) << eventQShort(event);
		os << setw(12) << eventQLong(event);
	}

	if (eventHasAB(event)) {
		os << setw(12) << eventA(event);
		os << setw(12) << eventAplusB(event);
	}

	if (eventHasPositionX(event)) {
		os << setw(12) << eventPositionX(event);
	}

	if (eventHasPositionY(event)) {
		os << setw(12) << eventPositionY(event);
	}

	// data[1]
	if (eventHasZeroCrossing(event)) {
		os << setw(12) << eventZeroCrossing(event);
	}

	if (eventHasFlags(event)) {

		ostringstream bos;

		uint16_t flags = eventFlags(event);
		for (int i = 15; i >= 0; --i) {
			uint16_t flag = (flags >> i) & 0x1;
			if (flag == 1) {
				bos << "1";
			}
			else {
				bos << "0";
			}
		}

		os << setw(18) << bos.str();
	}

	return os.str();
}

}
